#!/bin/bash

#EEXTR=/opt/essentia-2.1b4/bin/essentia_streaming_extractor_music
EEXTR=essentia_streaming_extractor_music
#PROF=/home/luser/source/javascript/LMSessentiaIntegration.addon/highlevel.profile
PROF=highlevel.profile
#DESTDIR=/home/luser/tmp/sshfs/tmp/ess-test/jsons
DESTDIR=""
SOURCEDIR=""
INFILE=""
#LMS="http://odroidhc2:9000/plugins/LMSessentiaIntegration/jsonupload.html"
LMS=""
FINDOPTS='-type f -a \( -iname \*.flac -o -iname \*.mp3 \)'
CURL=curl
JQ=jq
SEDSTR="q"
VBOSE=""
OPMODE=""
CURLLOG="/tmp/ei-curl.log"
CLOBBER=""

which "$CURL" &> /dev/null || { echo "Error: curl not found"; exit 1; }
which "$JQ" &> /dev/null || { echo "Error: jq not found"; exit 1; }
which readlink &> /dev/null || { echo "Error: readlink not found"; exit 1; }

while getopts "x:p:j:i:cj:s:m:vo:l:" opt; do
	case $opt in
		o)
		OPMODE="$OPTARG"
		;;
		x)
		EEXTR="$OPTARG"
		;;
		p)
		PROF="$OPTARG"
		;;
		j)
		DESTDIR="$OPTARG"
		;;
		i)
		INFILE="$OPTARG"
		;;
		m)
		SOURCEDIR="$OPTARG"
		;;
		s)
		SEDSTR="$OPTARG"
		echo "/some/file.flac"|sed -e "$SEDSTR" > /dev/null || { echo "ERROR: invalid sed command $OPTARG"; exit 1; }
		;;
		l)
		LMS="http://$OPTARG/plugins/LMSessentiaIntegration/jsonupload.html"
		;;
		c)
		CLOBBER="y"
		;;
		v)
		VBOSE="y"
		;;
		\?)
		echo "Invalid option: -$opt" >&2
		exit 1
		;;
		:)
		echo "Option -$opt requires an argument." >&2
		exit 1
		;;
	esac
done

echo "Operation: $OPMODE"
echo "Extractor: $EEXTR"
echo "Profile: $PROF"
echo "Data file dir: $DESTDIR"
[ -n "$SOURCEDIR" ] && echo "Music dir: $SOURCEDIR"
[ -n "$INFILE" ] && echo "Input file: $INFILE"
echo "LMS URL: $LMS"
echo "-------------------------------"

[ -z "$OPMODE" ] && { echo "ERROR: no operation specified (-o analyze/submit/all)"; exit 1; }
[ -d "$DESTDIR" ] || { echo "ERROR: data file dir $DESTDIR does not exist"; exit 1; }
[[ ( -n "$INFILE" && -n "$SOURCEDIR" ) && ( -z "$INFILE" && -z "$SOURCEDIR" ) ]] && { echo "ERROR: specify exactly one of -i or -m"; exit 1; }
[[ ("$OPMODE" == "analyze" || "$OPMODE" == "all") && ( -n "$INFILE" && ! -f "$INFILE" ) ]] && { echo "ERROR: input file $INFILE does not exist"; exit 1; }
[[ ("$OPMODE" == "analyze" || "$OPMODE" == "all") && ( -n "$SOURCEDIR" && ! -d "$SOURCEDIR" ) ]] && { echo "ERROR: source dir $SOURCEDIR does not exist"; exit 1; }
[[ ("$OPMODE" == "analyze" || "$OPMODE" == "all") && ! -f "$PROF" ]] && { echo "ERROR: profile file $PROF does not exist"; exit 1; }
[[ ("$OPMODE" == "analyze" || "$OPMODE" == "all") && ! -x "$EEXTR" ]] && { echo "ERROR: file $EEXTR is not executable"; exit 1; }
[[ ("$OPMODE" == "submit" || "$OPMODE" == "all") && -z "$LMS" ]] && { echo "ERROR: LMS server must be specified"; exit 1; }


function analyze_file {
	#local fullp="`realpath \"$1\"`"
	local fullp="`readlink -f \"$1\"`"
	[ -n "$VBOSE" ] && echo "- processing $1"
	local fullsed="$(echo $fullp|sed -e $SEDSTR)"
	local dest="$DESTDIR/`echo $fullsed|sed -e 's#/#_#g'`.json"
	[ -z "$CLOBBER" ] && [ -f "$dest" ] && echo "-- skipping $1 (destination file `basename \"${dest}\"` exists)" && return
	[ -f "$dest".lock ] && echo "-- skipping $1 (file locked)" && return
	touch "${dest}.lock"
	"$EEXTR" "$1" "$dest".tmp "$PROF"
	[ \! -f "$dest".tmp ] && echo "-- Error: extraction failed on $1" && rm  "${dest}.lock" && return
	"$JQ" ".metadata.tags.file_path = \"$fullsed\"" < "${dest}.tmp" > "${dest}"
	rm "${dest}.tmp" "${dest}.lock"
}

function analyze_input {
	[ -n "$VBOSE" ] && echo "* analyzing files read from $INFILE"
	while read -r i;  do
		if [ -f "$i" ]; then
			 analyze_file "$i"
		else
			echo "- Error: $i not found"
		fi
	done < "$INFILE"
}

function analyze_files {
	[ -n "$VBOSE" ] && echo "* analyzing files in $SOURCEDIR"
	IFS=$'\n'
	#for i in $(find "$SOURCEDIR" $FINDOPTS); do 
	for i in $(find "$SOURCEDIR" -type f -a \( -iname \*.mp3 -o -iname \*.flac -o -iname \*.ogg -o -iname \*.m4a \) | sort ); do 
		analyze_file "$i"
	done
}

function submit_files {
	[ -n "$VBOSE" ] && echo "* submitting files in $DESTDIR to $LMS"
	IFS=$'\n'
	for i in $(find "$DESTDIR" -type f -name \*.json | sort ); do 
		[ -n "$VBOSE" ] && echo "- processing $i"
		"$CURL" -fsS -H "Content-Type: application/json" -X POST -d "@${i}" -o "$CURLLOG" "$LMS"
		CURLRET=$?
		[[ $CURLRET -ne 0 ]] && echo "-- Error: curl failed with error $CURLRET. $CURLLOG may (or may not) have details." && continue
	done
}

if [[ "$OPMODE" == "analyze" || "$OPMODE" == "all" ]]; then
	if [ -n "$SOURCEDIR" ]; then
		analyze_files
	elif [ -n "$INFILE" ];  then
		analyze_input
	fi;
fi
if [[ "$OPMODE" == "submit"  || "$OPMODE" == "all" ]]; then
	submit_files
fi

exit 0
